#!/usr/bin/env python
# coding: utf-8

# In[5]:


def sierp_a(points, depth, dx, dy):
    '''Draw right across the top.'''
    if depth > 0:
        depth -= 1

        sierp_a(points, depth, dx, dy)
        draw_relative(points, dx, dy)
        sierp_b(points, depth, dx, dy)
        draw_relative(points, 2 * dx, 0)
        sierp_d(points, depth, dx, dy)
        draw_relative(points, dx, -dy)
        sierp_a(points, depth, dx, dy)

def sierp_b(points, depth, dx, dy):
    '''Draw down on the righ side.'''
    if depth > 0:
        depth -= 1

        sierp_b(points, depth, dx, dy)
        draw_relative(points, -dx, dy)
        sierp_c(points, depth, dx, dy)
        draw_relative(points, 0, 2 * dy)
        sierp_a(points, depth, dx, dy)
        draw_relative(points, dx, dy)
        sierp_b(points, depth, dx, dy)

def sierp_c(points, depth, dx, dy):
    '''Draw left across the bottom.'''
    if depth > 0:
        depth -= 1

        sierp_c(points, depth, dx, dy)
        draw_relative(points, -dx, -dy)
        sierp_d(points, depth, dx, dy)
        draw_relative(points, -2 * dx, 0)
        sierp_b(points, depth, dx, dy)
        draw_relative(points, -dx, dy)
        sierp_c(points, depth, dx, dy)

def sierp_d(points, depth, dx, dy):
    '''Up along the left side.'''
    if depth > 0:
        depth -= 1

        sierp_d(points, depth, dx, dy)
        draw_relative(points, dx, -dy)
        sierp_a(points, depth, dx, dy)
        draw_relative(points, 0, -2 * dy)
        sierp_c(points, depth, dx, dy)
        draw_relative(points, -dx, -dy)
        sierp_d(points, depth, dx, dy)

def sierpinski(points, depth, dx, dy):
    '''Draw a Sierpinski curve.'''
    x = 2 * dx
    y = dy

    sierp_a(points, depth, dx, dy)
    draw_relative(points, dx, dy)
    sierp_b(points, depth, dx, dy)
    draw_relative(points, -dx, dy)
    sierp_c(points, depth, dx, dy)
    draw_relative(points, -dx, -dy)
    sierp_d(points, depth, dx, dy)
    draw_relative(points, dx, -dy)


# In[6]:


def draw_relative(points, dx, dy):
    '''Add (dx, dy) to the last point in the points list.'''
    points.append(
        (points[-1][0] + dx, points[-1][1] + dy))


# In[7]:


import tkinter as tk
from tkinter import messagebox
import math

class SierpinskiApp:
    def __init__(self):
        self.window = tk.Tk()
        self.window.title('sierpinski_curve')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry('400x400')

        # Build the UI.
        frame = tk.Frame(self.window)
        frame.pack(side=tk.TOP, anchor=tk.W, padx=3, pady=3)

        label = tk.Label(frame, text='Depth:')
        label.pack(side=tk.LEFT, padx=3)
        self.depth_var = tk.StringVar(value='3')
        entry = tk.Entry(frame, textvariable=self.depth_var, width=3)
        entry.pack(side=tk.LEFT, padx=3)

        button = tk.Button(frame, text='Draw', width=10, command=self.draw)
        button.pack(side=tk.LEFT, padx=10)

        self.canvas = tk.Canvas(self.window, bg='white',
            borderwidth=2, relief=tk.SUNKEN)
        self.canvas.pack(padx=10, pady=(0, 10),
            side=tk.BOTTOM, fill=tk.BOTH, expand=True)

        # If the user presses Enter, draw.
        entry.bind('<Return>', lambda event: self.draw())

        # Display the window.
        self.window.focus_force()
        self.window.mainloop()

    def kill_callback(self):
        # Destroy the tkinter window.
        self.window.destroy()

    def draw(self):
        # Delete any previous drawing objects.
        self.canvas.delete(tk.ALL)

        # Get the desired depth.
        try:
            max_depth = 10
            depth = int(self.depth_var.get())
        except:
            messagebox.showinfo('Depth Error',
                f'Depth must be an integer between 1 and {max_depth}.')
            return
        if depth < 1 or depth > max_depth:
            messagebox.showinfo('Depth Error',
                f'Depth must be an integer between 1 and {max_depth}.')
            return

        # See how big we can make the curve.
        wid = self.canvas.winfo_width()
        hgt = self.canvas.winfo_height()

        # Compute dx and dy.
        margin = 10
        curve_width = min(wid, hgt) - 2 * margin
        num_divisions = (2 ** (depth + 2)) - 2
        dx = curve_width / num_divisions
        dy = dx
        start_x = (wid - curve_width) / 2 + dx
        start_y = (hgt - curve_width) / 2

        # Create the points list.
        points = [(start_x, start_y)]
        sierpinski(points, depth, dx, dy)

        # Draw the polygon.
        self.canvas.create_line(points, width=1, fill='black')
        print(f'# points: {len(points)}')

        # Optional: Draw a box around the curve.
        if False:
            x1 = start_x - dx
            y1 = start_y
            x2 = x1 + curve_width
            y2 = y1 + curve_width
            self.canvas.create_rectangle(x1, y1, x2, y2, width=1, fill='', outline='red')

        # Optional: Draw the starting point.
        if False:
            self.canvas.create_oval(start_x - 3, start_y - 3, start_x + 3, start_y + 3, width=1, fill='blue')


# In[8]:


SierpinskiApp()
print('Done')


# Depth 10: 4 or 5 seconds
# 
